;--------------------------------------------------------------------------------------------- 
; DECODIFICADOR DE CONTROLE REMOTO INFRAVERMELHO
;
; STATUS: FUNCIONA
;
;--------------------------------------------------------------------------------------------- 
; Este programa decodifica os controles remotos listados abaixo e mostra o resultado no LCD.
;
; TV SANYO
; TV TOSHIBA CT-8010
; DVD GRADIENTE CR-203
; CONSUL - Condicionador de Ar (nao e' o ar condicionado)
;
;--------------------------------------------------------------------------------------------- 
; HARDWARE
;
; Fsoc ................... 20MHZ externo obrigatorio
; RB0 .................... entrada do sinal do circuito detetor
; RB1 .................... LCD Data
; RB2 .................... LCD Clock
; RB3 .................... LED
;
; Veja o esquema do circuito detetor no arquivo ..\Detetor\rmtcltdeco.pgn deste projeto.
;
;--------------------------------------------------------------------------------------------- 
; SINAL DO CONTROLE REMOTO
;
; O sinal do controle remoto e' modulado numa portadora de aproximadamente
; 36KHZ que, antes de entrar neste programa, foi retirada pelo circuito
; detetor. Apos a retirada da portadora de 36KHZ, o sinal tem o seguinte
; formato:
;
;                               
;                               [<----------------- 32 bits ------------->]
;
;         HEADER                 DADO  VALOR  DADO    VALOR   DADO   VALOR
;    +--------------+           +-----+     +-----+          +-----+       +-----+
;    |    9,4ms     |           |0,8ms|0,8ms|0,8ms|   1,6ms  |0,8ms|       |0,8ms|  
;    |              |           |     |     |     |          |     |       |     |  
;    |              |           |     |     |     |          |     |       |     |  
;    |              |   BREAK   |     |     |     |          |     |       |     |   STOP
; ---+              +-----------+     +-----+     +----------+     +..   ..+     +------------
;                       2,0ms            0              1
;              
; Ao pressionar uma tecla do controle remoto este emitira o sinal acima onde:
;                 
; HEADER e' um nivel alto de 9,4ms (milisegundos) que indica o inicio da transmissao.
; BREAK  e' um nivel baixo de 2,0ms que indica que vai chegar um bit de dado.
; O nivel alto de 0,8ms apos o BREAK, indica que o nivel baixo de 0,8ms e' um bit de
; valor 0 (zero). Se o nivel baixo tiveer duracao de 1,6ms, entao o bit tera o 
; valor 1. 
; Apos enviar 32 bits de DADOS, sera enviado um nivel baixo de duracao muito 
; grande que significa fim da transmissao (STOP).
; Se a tecla do controle permanecer pressionada, o processo se repetira desde o
; envio do HEADER.
; 
; Para fazer a decodificacao do sinal, contaremos o numero de meios periodos de 
; duracao do sinal de DADO, isto 'e, 0,4ms. Assim, o HEADER de 9,4ms contera
; 23,5 meios periodos; o BREAK contera 5,0 meios periodos; DADO contera 2,0 meios
; periodos e VALOR contera 2,0 meios periodos.
;
;--------------------------------------------------------------------------------------------- 
	list      p=18F4520           ; list directive to define processor
	#include <p18F4520.inc>       ; processor specific variable definitions
	errorlevel  -302              ; suppress message 302 from list file
;
; ---------------------------------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; ---------------------------------------------------------------------------------------------
    CONFIG WDT   = OFF
	CONFIG MCLRE = ON
	CONFIG DEBUG = OFF
	CONFIG LVP   = OFF 
	CONFIG OSC   = HS 

;--------------------------------------------------------------------------------------------- 
  #include  "familias.inc"

;-------------------------------------------------------------------------
               UDATA    0x080
delay_temp     res      1
CLOCK          equ      20000000

rmtgcount      res      1              ; contador de 1/2 periodos
#define        rmtgtris TRISB          ; I/O
#define        rmtgport PORTB          ; PORTA 
rmtgbit        equ      0              ; BIT 
rmtgbuf        res      .4             ; buffer de leitura   

SR_1         RES          1             ; registrador auxiliar 1
SR_2         RES          1             ; registrador auxiliar 2
SR_3         RES          1             ; registrador auxiliar 3
SR_4         RES          1             ; registrador auxiliar 4
SR_5         RES          1             ; registrador auxiliar 5
SR_TIPO      res          1
LCD_COMMAND  RES          1
SR_CLK       equ          2             ; em TRISx este bit deve ser saida
SR_DAT       equ          1             ; em TRISx este bit deve ser saida
#define      SR_TRIS      TRISB
#define      SR_DATA      LATB,SR_DAT  ; DATA
#define      SR_CLOCK     LATB,SR_CLK  ; CLOCK
;
  #include "gpr.inc"


;-------------------------------------------------------------------------
INIT_VECTOR    CODE    0x00
               goto    inicio
INTERRUPT      CODE    0x08
               retfie
;-------------------------------------------------------------------------
MAIN           CODE    

  #include  "hexag.inc"
  #include  "delay_services.inc"
  #include  "rmctgenerico_services.inc"
  #include  "lcd_services.inc"

welcome     SR_4MVC      welc
welc        db        "HPSpin apresenta", 0

msg001      SR_4MVC      ms001
ms001       db        "Controle Remoto",0

msg004      SR_4MVC      ms004
ms004       db        "Pronto",0

msg005      SR_4MVC      ms005
ms005       db        "0x",0

msg006      SR_4MVC      ms006
ms006       db        " K=",0

msg007      SR_4MVC      ms007
ms007       db        "Codigo invalido",0

  #include "hex2ascii.inc"

           LCD_LINES 0216

;-------------------------------------------------------------------------
inicio         movlw   0x0F                      ;  
               movwf   ADCON1                    ; todas as portas sao I/O digital

               clrf    LATA
               clrf    LATB

               movlw   0xFF
               movwf   TRISA
               movlw   b'00000111'                ; Tx + Rx + RMTBIT
               movwf   TRISB

               movlw   .10
               movwf   R12
pisca          movlw   .8
               movwf   LATB
               movlw   .100
               call    delay_ms
               movlw   .0
               movwf   LATB
               movlw   .100
               call    delay_ms
               decfsz  R12, f
               goto    pisca

               call    SR_4BITS
               call    SR_LCDCLEAR
               SET_CURSOR .1, .1, "i"
               call    welcome
               movlw   .1
               call    tempo

               SET_CURSOR .2, .1, "i"
               call    msg001                    ; msg="Controle Remoto"
               movlw   .1
               call    tempo

               call    SR_LCDCLEAR

ready          SET_CURSOR .1, .1, "i"
               call    msg004                    ; msg="Promto"

               call    rmct_generico             ; le o controle remoto

; --------------------------------------------------------------------------
; Mostra o resultado da leitura do controle remoto
; --------------------------------------------------------------------------
stop_run       call    SR_LCDCLEAR
               SET_CURSOR .2, .1, "i"
               call    msg005                    ; msg="=x"
               movf    rmtgbuf, w
               call    shexa
               movf    rmtgbuf+.1, w
               call    shexa
               movf    rmtgbuf+.2, w
               call    shexa
               movf    rmtgbuf+.3, w
               call    shexa
               call    msg006                    ; msg=" K="
               comf    rmtgbuf, w                ; complementa o codigo da tecla
               subwf   rmtgbuf+.1, w             ; compara com o verificador
               btfss   STATUS, Z                 ; codigos semelhantes???
               goto    erro                      ; NAO --> erro
               movf    rmtgbuf+.1, w             ; pega o codigo da tecla
               call    shexa                     ; mostra o codigo da tecla
               movlw   .1
               call    tempo
               goto    ready
; --------------------------------------------------------------------------
erro           call    SR_LCDCLEAR
               SET_CURSOR .1, .1, "i"
               call    msg007                    ; msg="Codigo invalido"
               movlw   .1
               call    tempo
               call    SR_LCDCLEAR
               goto    ready

; --------------------------------------------------------------------------
shexa          call    HEXAG
               movf    R1, w
               call    SR_SENDW
               movf    R0, w
               call    SR_SENDW
               return
; --------------------------------------------------------------------------
tempo          movwf   R4
               rlcf    R4, f
               rlcf    R4, f
tx1            movlw   .250
               call    delay_ms
               decfsz  R4, f
               goto    tx1
               return
; --------------------------------------------------------------------------
; 



               end
